@@ -49,6 +49,7 @@
 struct GnomeSettingsManagerPrivate
 {
         DBusGConnection            *connection;
+        GConfClient                *gconf_client;
         char                       *settings_prefix;
         GSList                     *plugins;
 };
@@ -157,24 +158,21 @@ _load_file (GnomeSettingsManager *manager,
 {
         GnomeSettingsPluginInfo *info;
         char                    *key_name;
-        GConfClient             *client;
         int                      priority;
         GError                  *error;
         GSList                  *l;
 
         g_debug ("Loading plugin: %s", filename);
-        gnome_settings_profile_start (NULL);
+        gnome_settings_profile_start ("%s", filename);
 
         info = gnome_settings_plugin_info_new_from_file (filename);
         if (info == NULL) {
                 goto out;
         }
 
-        gnome_settings_profile_start ("seeing if already loaded");
         l = g_slist_find_custom (manager->priv->plugins,
                                  info,
                                  (GCompareFunc) compare_location);
-        gnome_settings_profile_end ("seeing if already loaded");
         if (l != NULL) {
                 goto out;
         }
@@ -187,21 +185,17 @@ _load_file (GnomeSettingsManager *manager,
         g_signal_connect (info, "deactivated",
                           G_CALLBACK (on_plugin_deactivated), manager);
 
-        gnome_settings_profile_start ("setting active property");
         key_name = g_strdup_printf ("%s/%s/active",
                                     manager->priv->settings_prefix,
                                     gnome_settings_plugin_info_get_location (info));
         gnome_settings_plugin_info_set_enabled_key_name (info, key_name);
         g_free (key_name);
-        gnome_settings_profile_end ("setting active property");
 
-        gnome_settings_profile_start ("setting priority property");
         key_name = g_strdup_printf ("%s/%s/priority",
                                     manager->priv->settings_prefix,
                                     gnome_settings_plugin_info_get_location (info));
-        client = gconf_client_get_default ();
         error = NULL;
-        priority = gconf_client_get_int (client, key_name, &error);
+        priority = gconf_client_get_int (manager->priv->gconf_client, key_name, &error);
         if (error == NULL) {
                 if (priority > 0) {
                         gnome_settings_plugin_info_set_priority (info, priority);
@@ -210,15 +204,13 @@ _load_file (GnomeSettingsManager *manager,
                 g_error_free (error);
         }
         g_free (key_name);
-        g_object_unref (client);
-        gnome_settings_profile_end ("setting priority property");
 
  out:
         if (info != NULL) {
                 g_object_unref (info);
         }
 
-        gnome_settings_profile_end (NULL);
+        gnome_settings_profile_end ("%s", filename);
 }
 
 static void
@@ -235,7 +227,7 @@ _load_dir (GnomeSettingsManager *manager,
         error = NULL;
         d = g_dir_open (path, 0, &error);
         if (d == NULL) {
-                g_warning (error->message);
+                g_warning ("%s", error->message);
                 g_error_free (error);
                 return;
         }
@@ -345,6 +337,15 @@ gnome_settings_manager_start (GnomeSettingsManager *manager,
                 goto out;
         }
 
+        manager->priv->gconf_client = gconf_client_get_default ();
+
+        gnome_settings_profile_start ("preloading gconf keys");
+        gconf_client_add_dir (manager->priv->gconf_client,
+                              manager->priv->settings_prefix,
+                              GCONF_CLIENT_PRELOAD_RECURSIVE,
+                              NULL);
+        gnome_settings_profile_end ("preloading gconf keys");
+
         _load_all (manager);
 
         ret = TRUE;
@@ -379,6 +380,12 @@ gnome_settings_manager_stop (GnomeSettingsManager *manager)
 #endif
 
         _unload_all (manager);
+
+        gconf_client_remove_dir (manager->priv->gconf_client,
+                                 manager->priv->settings_prefix,
+                                 NULL);
+        g_object_unref (manager->priv->gconf_client);
+        manager->priv->gconf_client = NULL;
 }
 
 static void
