@@ -165,7 +165,6 @@ numlock_xkb_init (GsdKeyboardManager *manager)
         gboolean have_xkb;
         int opcode, error_base, major, minor;
 
-        gdk_error_trap_push ();
         have_xkb = XkbQueryExtension (dpy,
                                       &opcode,
                                       &manager->priv->xkb_event_base,
@@ -184,9 +183,6 @@ numlock_xkb_init (GsdKeyboardManager *manager)
                 g_warning ("XKB extension not available");
         }
 
-        XSync (dpy, FALSE);
-        gdk_error_trap_pop ();
-
         manager->priv->have_xkb = have_xkb;
 }
 
@@ -374,9 +370,8 @@ apply_settings (GConfClient        *client,
         gdk_error_trap_pop ();
 }
 
-gboolean
-gsd_keyboard_manager_start (GsdKeyboardManager *manager,
-                            GError            **error)
+static gboolean
+start_keyboard_idle_cb (GsdKeyboardManager *manager)
 {
         GConfClient *client;
 
@@ -412,6 +407,19 @@ gsd_keyboard_manager_start (GsdKeyboardManager *manager,
 
         gnome_settings_profile_end (NULL);
 
+        return FALSE;
+}
+
+gboolean
+gsd_keyboard_manager_start (GsdKeyboardManager *manager,
+                            GError            **error)
+{
+        gnome_settings_profile_start (NULL);
+
+        g_idle_add ((GSourceFunc) start_keyboard_idle_cb, manager);
+
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
